
##Time series Model

library(dplyr)
library(rugarch)
library(tseries)
library(ggplot2)
library(cowplot)


TS_whole_data = read.csv("/Users/tongshihao/Desktop/TS_whole_data.csv")
#Chage Y into the difference between curve_based_price and curve_based_price_last1
#since at time t, traders are only accessible to the curve price yesterday, so it make more
#sence than modeling the difference with today's curve price

TS_whole_data$curve_trade_diff <- TS_whole_data$trade_price - TS_whole_data$curve_based_price_last1
plot(head(TS_whole_data$curve_trade_diff, 20000),ylab = "Difference")
hist(TS_whole_data$curve_trade_diff, xlab = "Difference")
summary(TS_whole_data$curve_trade_diff)
var(TS_whole_data$curve_trade_diff)
#Comments: The difference are centered around zero with small variance. 
#It is not necessary to do transformation. 

#Illustration 
count_max_by_id <- function(x){ #x is the dataset
  count = x %>% group_by(x$bond_id) %>% count()
  count = as.data.frame(count)
  names(count)[1] <- "id"
  max <-  max(count$n)
  max_id = count$id[count$n == max]
  return(c(max,max_id)) #c(number of recordings, bond_id)
} #get c(#of recording, bond_id)
ts_creation_diff<- function(x,id){ #x is dataset, id is the bond_id 
  ts = ts(x$curve_trade_diff[x$bond_id == id])
  return(ts)
} #Create ts of curve_trade_diff with given id and dataset
num_rec_by_id = as.data.frame(TS_whole_data %>%group_by(bond_id) %>% count()) #Num of recording of each bond
ts_1 = ts_creation_diff(TS_whole_data, count_max_by_id(TS_whole_data)[2])
#data_ts_1 = TS_whole_data[TS_whole_data$bond_id == count_max_by_id(TS_whole_data)[2],]
ts_2 = ts_creation_diff(TS_whole_data, 5595)
#data_ts_2 = TS_whole_data[TS_whole_data$bond_id == 5595,]
plot(ts_1,ylab = "Difference")
plot(ts_2,ylab = "Difference")
acf(ts_1)
acf(ts_2)
pacf(ts_1)
pacf(ts_2)

#acf does not provide useful information to determine the order of ARMA.
#and the ts plot shows the variance are definitely unequal, so the conventional ARMA model
#is not a good candadites for ts model. To deal with the extreme changes we 
#consider the ARMA(p1,q1)-GARCH(p2,q2) which is accepted popularly in 
#dealing with finanical data. 

##Strategy
#The main purpose is to determine
#   a. The hyper parameter of ARMA and GARCH
#   b. The external regressor to be included, if improve the performance
#The strategy is 
#   a. Try possible combination of p1,q1,p2,q2.
#   b. Extend the model to predit for each bond.
#   c. Adding external regressors to the model.
#   d. Extend the model to predit for each bond.
#The case for external regressor is dealt with seperately since the number of constrain changes. 

##Possible combinations
#In the test set, since each bond have only 10 previous trading information
#avilable, so that means, in order the predict all possible parameter, at most
#the degree of freedom is 10. However in order to obtain a, at least reasonble,
#decent prediction, we determine to restrict the the num of p1, q1, p2, q2 sum
#to be at most 6 taking into consideration the \sigma^2 which is also need to be 
#fitted. 

#Note: Pure ARMA are not involved. (i.e p1 and q1 not 0 simutaneouly).

pq_combination = list(
  c(0,0,1,0),c(0,0,1,1),c(0,0,1,2),c(0,0,1,3),c(0,0,1,4),c(0,0,1,5),
  c(0,0,2,0),c(0,0,2,1),c(0,0,2,2),c(0,0,2,3),c(0,0,2,4),c(0,0,3,0),
  c(0,0,3,1),c(0,0,3,2),c(0,0,3,3),c(0,0,4,0),c(0,0,4,1),c(0,0,4,2),
  c(0,0,5,1),c(0,0,5,0),c(0,0,6,0),
  c(1,0,1,0),c(1,0,1,1),c(1,0,1,2),c(1,0,1,3),c(1,0,1,4),
  c(1,0,2,0),c(1,0,2,1),c(1,0,2,2),c(1,0,2,3),c(1,0,3,0),
  c(1,0,3,1),c(1,0,3,2),c(1,0,4,0),c(1,0,4,1),
  c(1,0,5,0),
  c(2,0,1,0),c(2,0,1,1),c(2,0,1,2),c(2,0,1,3),
  c(2,0,2,0),c(2,0,2,1),c(2,0,2,2),c(2,0,3,0),
  c(2,0,4,0),
  c(3,0,1,0),c(3,0,1,1),c(3,0,1,2),
  c(3,0,2,0),c(3,0,2,1),c(3,0,3,0),
  c(4,0,1,0),c(4,0,1,1),
  c(4,0,2,0),
  c(5,0,1,0), 
  c(1,1,1,0),c(1,1,1,1),c(1,1,1,2),c(1,1,1,3),
  c(1,1,2,0),c(1,1,2,1),c(1,1,2,2),c(1,1,3,0),
  c(1,1,4,0),
  c(2,1,1,0),c(2,1,1,1),c(2,1,1,2),
  c(2,1,2,0),c(2,1,2,1),c(2,1,3,0),
  c(3,1,1,0),c(3,1,1,1),
  c(3,1,2,0),
  c(4,1,1,0),
  c(1,2,1,0),c(1,2,1,1),
  c(1,2,2,0),c(1,2,2,1),c(1,2,3,0),
  c(2,2,1,0),c(2,2,1,1),
  c(2,2,2,0),
  c(3,2,1,0),
  c(1,3,1,0),c(1,3,1,1),
  c(1,3,2,0),c(2,3,1,0),
  c(1,4,1,0)
)


##Testing Error
WMSE_res <- function(res,w){
  sum(w*abs(res))/sum(w)
}
MSE_res <- function(res){
  sqrt(sum(abs(res)^2)/(length(res)-1))
}
intervalScorePlus=function(predObj,actual,alpha){ 
  n=nrow(predObj)
  ilow=(actual<predObj[,2]) # underestimation
  ihigh=(actual>predObj[,3]) # overestimation
  sumlength=sum(predObj[,3]-predObj[,2]) # sum of lengths of prediction intervals
  sumlow=sum(predObj[ilow,2]-actual[ilow])*2/alpha
  sumhigh=sum(actual[ihigh]-predObj[ihigh,3])*2/alpha
  avglength=sumlength/n
  IS=(sumlength+sumlow+sumhigh)/n # average length + average under/over penalties
  cover=mean(actual>= predObj[,2] & actual<=predObj[,3])
  summ=c(alpha,avglength,IS,cover)
  summ
} 

##Model Fitting 
#For each of the combination above, fit a model for each bond and then
#go forward. We use 60% of each bond to fit the model and use 40% to do
#prediction and validation

#With No external Regresson
model_fitting <- function(hyper,train){ #hyper is a 4 dimentional vector (p1,q1,p2,q2)
                                        #train is a ts/vector used for training
  spec <- ugarchspec(variance.model = list(model = "sGARCH", 
                                            garchOrder = hyper[3:4], 
                                            submodel = NULL, 
                                            external.regressors = NULL, 
                                            variance.targeting = TRUE), 
                      
                      mean.model     = list(armaOrder = hyper[1:2], 
                                            external.regressors = NULL,
                                            distribution = "std", 
                                            start.pars = list(), 
                                            fixed.pars = list()))
  garch <- ugarchfit(spec = spec, data = train ,solver ='hybrid',solver.control = list(trace=0))
  return(garch)
}#retrun model
                                          #Solver is set tobe 'hybrid' inorder for 
                                          #convergence (not sure what does this mean)
 
prediction_ts <- function(model, forwardstep){ #model is the arma-grach model fitted
                                                  #ts is the historical ts
                                                  #forwardstep is the number of steps to predict
  forecast = ugarchforecast(model, n.ahead = forwardstep)
  return(forecast)
}#return forcasted s4 object
                                                      #notice in this case the ts should be a adjuctive ts of the seris used to fit the model
determin_trian_test_size <- function(id){
  recod_by_id = num_rec_by_id[num_rec_by_id$bond_id == id,]$n
  ntrain = round(0.6*recod_by_id)
  ntest = recod_by_id - ntrain
  size = c(ntrain,ntest)
  return(size)
} #return c(ntrain,ntest)
#test: determin_trian_test_size(9563) WORKS
split_ts <- function(tsdata,ntest){
  ntrain = length(tsdata) - ntest
  ts_train = tsdata[1:ntrain]
  ts_test = tsdata[(ntrain+1):length(tsdata)]
  return(list(ts_train,ts_test))
}#return list(train_ts,test_ts) y values


#Fitting process
#give pq,iterate over bond_id
bond_ids = num_rec_by_id$bond_id[num_rec_by_id$n >= 100]


performance <- function(hype){
    listof_wesp_in = c()
    listof_wesp_out = c()
    listof_rmse_in = c()
    listof_rmse_out = c()
    dataframe_forISplus = c(1,1,1,1)
    for(i in 1:length(bond_ids)) {
      bondid = bond_ids[i]
      bond_subset = TS_whole_data[TS_whole_data$bond_id == bondid,]
      nforward = determin_trian_test_size(bond_ids[i])[2] 
      ntrain = dim(bond_subset)[1] - nforward
      c = split_ts(bond_subset$curve_trade_diff,nforward)
      ts_test = as.numeric(unlist(c[2])) 
      ts_train = as.numeric(unlist(c[1])
                            )
      skip_to_next <- FALSE
      tryCatch(model_fitting(hype, ts_train), error = function(e) {skip_to_next <<- TRUE})
      if(skip_to_next) { next }  
      skip_to_next <- FALSE
      model = model_fitting(hype, ts_train)#s4 object
      tryCatch(prediction_ts(model,nforward), error = function(e) {skip_to_next <<- TRUE})
      if(skip_to_next) { next }    
      prediction = prediction_ts(model,nforward) #s4 object
      
        outsample_ressidual =  ts_test - prediction@forecast$seriesFor
      
      #performance evaluation
      ##WESP
      wesp_in = WMSE_res(model@fit$residuals,head(bond_subset$re_weight,ntrain))
      listof_wesp_in = c(listof_wesp_in, wesp_in)
      wesp_out = WMSE_res(outsample_ressidual,tail(bond_subset$re_weight,nforward))
      listof_wesp_out = c(listof_wesp_out, wesp_out)
      #rmse
      rmse_in = MSE_res(model@fit$residuals)
      listof_rmse_in = c(listof_rmse_in, rmse_in)
      rmse_out = MSE_res(outsample_ressidual)
      listof_rmse_out = c(listof_rmse_out, rmse_out)
      #IS, coverage rate, average length
      dof = length(ts_test)-1
      qt_.5 = -qt(0.25, df=dof)    
      qt_.8 = -qt(0.1, df=dof)
      predicted_ts=  as.vector(prediction@forecast$seriesFor)
      predicted_lb_ts_.5 =  as.vector(prediction@forecast$seriesFor)-qt_.5*prediction@forecast$sigmaFor
      predicted_up_ts_.5 = as.vector(prediction@forecast$seriesFor)+qt_.5*prediction@forecast$sigmaFor
      predicted_lb_ts_.8 =  as.vector(prediction@forecast$seriesFor)-qt_.8*prediction@forecast$sigmaFor
      predicted_up_ts_.8 = as.vector(prediction@forecast$seriesFor)+qt_.8*prediction@forecast$sigmaFor
      predobject_.5 = cbind(predicted_ts,predicted_lb_ts_.5,predicted_up_ts_.5)
      predobject_.8 = cbind(predicted_ts,predicted_lb_ts_.8,predicted_up_ts_.8)
      a = intervalScorePlus(predobject_.5,ts_test,0.5)
      b = intervalScorePlus(predobject_.8,ts_test,0.8)
      dataframe_forISplus <- rbind(dataframe_forISplus,a,b)
    }
    dataframe_forISplus <- dataframe_forISplus[-1,]
    dataframe_forISplus <- as.data.frame(dataframe_forISplus)
    colnames(dataframe_forISplus) <- c("alpha", "Averagelength", "IS", "Coverage_Rate")
    rownames(dataframe_forISplus) <- 1:dim(dataframe_forISplus)[1]
    mean_.5_avelength = mean(dataframe_forISplus$Averagelength[dataframe_forISplus$alpha == 0.5])
    mean_.8_avelength = mean(dataframe_forISplus$Averagelength[dataframe_forISplus$alpha == 0.8])
    mean_.5_IS = mean(dataframe_forISplus$IS[dataframe_forISplus$alpha == 0.5])
    mean_.8_IS = mean(dataframe_forISplus$IS[dataframe_forISplus$alpha == 0.8])
    mean_.5_covrate = mean(dataframe_forISplus$Coverage_Rate[dataframe_forISplus$alpha == 0.5])
    mean_.8_covrate = mean(dataframe_forISplus$Coverage_Rate[dataframe_forISplus$alpha == 0.8])
    summ = c(mean_.5_avelength,mean_.8_avelength,mean_.5_IS,mean_.8_IS,mean_.5_covrate,mean_.8_covrate)
    namm = c("AverageLength_0.5","AverageLength_0.8","IS_0.5","IS_0.8","CoverageRate_0.5","CoverageRate_0.8")
    framm = as.data.frame(rbind(namm,summ))
    colnames(framm) <- framm[1,]
    framm <- framm[-1,]
    mean_wesp_in = mean(listof_wesp_in)
    mean_wesp_out = mean(listof_wesp_out)
    mean_rmse_in = mean(listof_rmse_in)
    mean_rmse_out = mean(listof_rmse_out)
    name = c("mean_wesp_in","mean_wesp_out","mean_rmse_in","mean_rmse_out")
    values = c(mean_wesp_in,mean_wesp_out,mean_rmse_in,mean_rmse_out)
    frame = as.data.frame(rbind(name,values), header = F)
    colnames(frame) <- frame[1,]
    frame <- frame[-1,]
    whole_frame <- cbind(frame,framm) 
    return(whole_frame)
}

nm = c("mean_wesp_in","mean_wesp_out","mean_rmse_in","mean_rmse_out","AverageLength_0.5","AverageLength_0.8","IS_0.5","IS_0.8","CoverageRate_0.5","CoverageRate_0.8")
cc = c(1,1,1,1,1,1,1,1,1,1)
final_data_frame = as.data.frame(rbind(nm,cc))
colnames(final_data_frame) <- final_data_frame[1,]
final_data_frame <- final_data_frame[-1,]

for(i in 1:92){
  hyper = as.vector(unlist(pq_combination[i]))
  a = performance(hyper)[1,]
  final_data_frame <- rbind(final_data_frame,a)
}

final_data_frame <- final_data_frame[-1,]



#Best model, no external regressor
min_WESP = min(final_data_frame$mean_wesp_out)
min_WESP_pqcomb = final_data_frame[final_data_frame$mean_wesp_out == min_WESP,] #row 82
min_WESP_pqcomb = pq_combination[82] #(2,2,1,1)

min_rmse =  min(final_data_frame$mean_rmse_out)
min_rmse_pqcomb = final_data_frame[final_data_frame$mean_rmse_out == min_rmse,] #row 84
min_WESP_pqcomb = pq_combination[84] #(3,2,1,0)

min_IS0.5 =  min(final_data_frame$IS_0.5)
min_IS0.5_pqcomb = final_data_frame[final_data_frame$IS_0.5 == min_IS0.5,] #row 84
min_IS0.5_pqcomb = pq_combination[84] #(3,2,1,0)

min_IS0.8 =  min(final_data_frame$IS_0.8)
min_IS0.8_pqcomb = final_data_frame[final_data_frame$IS_0.8 == min_IS0.8,] #row 84
min_IS0.8_pqcomb = pq_combination[84] #(3,2,1,0)


min_avlength =  min(final_data_frame$AverageLength_0.8)
min_avlength_pqcomb = final_data_frame[final_data_frame$AverageLength_0.8 == min_avlength,] #row 88
min_avlength_pqcomb = pq_combination[88] #(2,3,1,0)

min_coverate = min(final_data_frame$CoverageRate_0.8)
min_min_coverate = final_data_frame[final_data_frame$CoverageRate_0.8 == min_coverate,] #row 82
min_avlength_pqcomb = pq_combination[82] #(2,2,1,1)


#best choice: no external regressor
best1 = final_data_frame[88,]#(2,3,1,0)
best2 = final_data_frame[82,]#(2,2,1,1)




#Illustration of final_dafinal_data_frame2ta_frame
#p1 <- ggplot(data=final_data_frame, aes(x ="Mean WESP", y = as.numeric(mean_wesp_out))) + geom_boxplot(fill = "#FFDB6D") +theme(axis.title.x=element_blank(),
                                                                                                                #axis.title = element_blank(),
                                                                                                                #axis.ticks.x=element_blank()) + ylab("Scores")
#p2 <- ggplot(data=final_data_frame, aes(x ="Mean RMSE", y = as.numeric(mean_rmse_out))) + geom_boxplot(fill = "#FFDB6D")+theme(axis.title.x=element_blank(),
                                                                                                              # axis.title = element_blank(),                                                                                                              axis.title.y = element_blank(),
#                                                                                                              
#                                                                                                               axis.ticks.x=element_blank())
# p3 <- ggplot(data=final_data_frame, aes(x ="Mean IS, alpha =0.5", y = as.numeric(IS_0.5))) + geom_boxplot(fill = "#FFDB6D")+theme(axis.title.x=element_blank(),
#                                                                                                                   axis.title = element_blank(),
#                                                                                                                   axis.title.y = element_blank(),
#                                                                                                                   axis.ticks.x=element_blank())
# p4 <- ggplot(data=final_data_frame, aes(x ="Mean IS, alpha =0.8", y = as.numeric(IS_0.8))) + geom_boxplot(fill = "#FFDB6D")+theme(axis.title.x=element_blank(),
#                                                                                                                   axis.title = element_blank(),
#                                                                                                                   axis.title.y = element_blank(),
#                                                                                                                   axis.ticks.x=element_blank())
# p5 <- ggplot(data=final_data_frame, aes(x ="Mean AL0.8", y = as.numeric(AverageLength_0.8))) + geom_boxplot(fill = "#FFDB6D")+theme(axis.title.x=element_blank(),
#                                                                                                                             axis.title = element_blank(),
#                                                                                                                             axis.title.y = element_blank(),
#                                                                                                                             axis.ticks.x=element_blank())
# p6 <- ggplot(data=final_data_frame, aes(x ="Mean CR0.8", y = as.numeric(CoverageRate_0.8))) + geom_boxplot(fill = "#FFDB6D")+theme(axis.title.x=element_blank(),
#                                                                                                                            axis.title = element_blank(),
#                                                                                                                            axis.title.y = element_blank(),
#                                                                                                                            axis.ticks.x=element_blank())
# p7 <- ggplot(data=final_data_frame, aes(x ="Mean AL0.5", y = as.numeric(AverageLength_0.5))) + geom_boxplot(fill = "#FFDB6D")+theme(axis.title.x=element_blank(),
#                                                                                                                            axis.title = element_blank(),
#                                                                                                                            axis.title.y = element_blank(),
#                                                                                                                            axis.ticks.x=element_blank())
# p8 <- ggplot(data=final_data_frame, aes(x ="Mean CR0.5", y = as.numeric(CoverageRate_0.5))) + geom_boxplot(fill = "#FFDB6D")+theme(axis.title.x=element_blank(),
#                                                                                                                                axis.title = element_blank(),
#                                                                                                                                axis.title.y = element_blank(),
#                                                                                                                                axis.ticks.x=element_blank())
# plot_grid(p1, p2, p3, p4, p5, p6, p7, p8, labels = "AUTO", ncol = 8)








#With esternal regressor
model_fitting_external <- function(hyper,train,regressor){ #hyper is a 4 dimentional vector (p1,q1,p2,q2)
  #train is a ts/vector used for training and regressor is a ntrain dimionsinal vector
  spec <- ugarchspec(variance.model = list(model = "sGARCH", 
                                           garchOrder = hyper[3:4], 
                                           submodel = NULL, 
                                           external.regressors = NULL, 
                                           variance.targeting = TRUE), 
                     
                     mean.model     = list(armaOrder = hyper[1:2], 
                                           external.regressors = matrix(regressor),
                                           distribution = "std", 
                                           start.pars = list(), 
                                           fixed.pars = list()))
  garch <- ugarchfit(spec = spec, data = train ,solver ='hybrid',solver.control = list(trace=0))
  return(garch)
}

prediction_external_reg <- function(model, forwardstep, regressor){ #model is the arma-grach model fitted
  #ts is the historical ts
  #forwardstep is the number of steps to predict
  #regressor is a list object contaning the esternal data
  forecast = ugarchforecast(model, n.ahead = forwardstep, external.forecasts = as.list(regressor))
  return(forecast)
}

performance_external_reg <- function(hype,num){ #num is an char specifies with regressor to use
                                                #b is trade_size
                                                #c is reporting_delay
                                                #d is time_to_maturity
  listof_wesp_in = c()
  listof_wesp_out = c()
  listof_rmse_in = c()
  listof_rmse_out = c()
  dataframe_forISplus = c(1,1,1,1)
  for(i in 1:length(bond_ids)) {
    bondid = bond_ids[i]
    bond_subset = TS_whole_data[TS_whole_data$bond_id == bondid,]
    nforward = determin_trian_test_size(bond_ids[i])[2] 
    ntrain = dim(bond_subset)[1] - nforward
    c = split_ts(bond_subset$curve_trade_diff,nforward)
    ts_test = as.numeric(unlist(c[2])) 
    ts_train = as.numeric(unlist(c[1]))
    regress_train = c()
    regress_test = c()
    if(num == "b") {
      regress_train = bond_subset$trade_size[1:ntrain]
      regress_test  = bond_subset$trade_size[(ntrain+1):dim(bond_subset)[1]]
    }
    if(num == "c") {
      regress_train = bond_subset$reporting_delay[1:ntrain]
      regress_test  = bond_subset$reporting_delay[(ntrain+1):dim(bond_subset)[1]]
    }
    if(num == "d") {
      regress_train = bond_subset$time_to_maturity[1:ntrain]
      regress_test  = bond_subset$time_to_maturity[(ntrain+1):dim(bond_subset)[1]]
    }
    model = model_fitting_external(hype, ts_train, regress_train)#s4 object
    prediction = prediction_external_reg(model,nforward,regress_test) #s4 object
    outsample_ressidual =  ts_test - prediction@forecast$seriesFor
    
    #performance evaluation
    ##WESP
    wesp_in = WMSE_res(model@fit$residuals,head(bond_subset$re_weight,ntrain))
    listof_wesp_in = c(listof_wesp_in, wesp_in)
    wesp_out = WMSE_res(outsample_ressidual,tail(bond_subset$re_weight,nforward))
    listof_wesp_out = c(listof_wesp_out, wesp_out)
    #rmse
    rmse_in = MSE_res(model@fit$residuals)
    listof_rmse_in = c(listof_rmse_in, rmse_in)
    rmse_out = MSE_res(outsample_ressidual)
    listof_rmse_out = c(listof_rmse_out, rmse_out)
    #IS, coverage rate, average length
    dof = length(ts_test)-1
    qt_.5 = -qt(0.25, df=dof)    
    qt_.8 = -qt(0.1, df=dof)
    predicted_ts=  as.vector(prediction@forecast$seriesFor)
    predicted_lb_ts_.5 =  as.vector(prediction@forecast$seriesFor)-qt_.5*prediction@forecast$sigmaFor
    predicted_up_ts_.5 = as.vector(prediction@forecast$seriesFor)+qt_.5*prediction@forecast$sigmaFor
    predicted_lb_ts_.8 =  as.vector(prediction@forecast$seriesFor)-qt_.8*prediction@forecast$sigmaFor
    predicted_up_ts_.8 = as.vector(prediction@forecast$seriesFor)+qt_.8*prediction@forecast$sigmaFor
    predobject_.5 = cbind(predicted_ts,predicted_lb_ts_.5,predicted_up_ts_.5)
    predobject_.8 = cbind(predicted_ts,predicted_lb_ts_.8,predicted_up_ts_.8)
    a = intervalScorePlus(predobject_.5,ts_test,0.5)
    b = intervalScorePlus(predobject_.8,ts_test,0.8)
    dataframe_forISplus <- rbind(dataframe_forISplus,a,b)
  }
  dataframe_forISplus <- dataframe_forISplus[-1,]
  dataframe_forISplus <- as.data.frame(dataframe_forISplus)
  colnames(dataframe_forISplus) <- c("alpha", "Averagelength", "IS", "Coverage_Rate")
  rownames(dataframe_forISplus) <- 1:dim(dataframe_forISplus)[1]
  mean_.5_avelength = mean(dataframe_forISplus$Averagelength[dataframe_forISplus$alpha == 0.5])
  mean_.8_avelength = mean(dataframe_forISplus$Averagelength[dataframe_forISplus$alpha == 0.8])
  mean_.5_IS = mean(dataframe_forISplus$IS[dataframe_forISplus$alpha == 0.5])
  mean_.8_IS = mean(dataframe_forISplus$IS[dataframe_forISplus$alpha == 0.8])
  mean_.5_covrate = mean(dataframe_forISplus$Coverage_Rate[dataframe_forISplus$alpha == 0.5])
  mean_.8_covrate = mean(dataframe_forISplus$Coverage_Rate[dataframe_forISplus$alpha == 0.8])
  summ = c(mean_.5_avelength,mean_.8_avelength,mean_.5_IS,mean_.8_IS,mean_.5_covrate,mean_.8_covrate)
  namm = c("AverageLength_0.5","AverageLength_0.8","IS_0.5","IS_0.8","CoverageRate_0.5","CoverageRate_0.8")
  framm = as.data.frame(rbind(namm,summ))
  colnames(framm) <- framm[1,]
  framm <- framm[-1,]
  mean_wesp_in = mean(listof_wesp_in)
  mean_wesp_out = mean(listof_wesp_out)
  mean_rmse_in = mean(listof_rmse_in)
  mean_rmse_out = mean(listof_rmse_out)
  name = c("mean_wesp_in","mean_wesp_out","mean_rmse_in","mean_rmse_out")
  values = c(mean_wesp_in,mean_wesp_out,mean_rmse_in,mean_rmse_out)
  frame = as.data.frame(rbind(name,values), header = F)
  colnames(frame) <- frame[1,]
  frame <- frame[-1,]
  whole_frame <- cbind(frame,framm) 
  return(whole_frame)
}


#Given best model (3,2,1,0) and (2,3,1,0) we have to sacrifice more dof to 
#add a new external regressors. So we tried to reduce p1 and q1 by 1. 

performance_external_reg(c(2,2,1,0),"b")
performance_external_reg(c(2,2,1,0),"c")
performance_external_reg(c(2,2,1,0),"d")

performance_external_reg(c(3,1,1,0),"b")
performance_external_reg(c(3,1,1,0),"c")
performance_external_reg(c(3,1,1,0),"d")


final_data_frame_esternal <- rbind(performance_external_reg(c(2,2,1,0),"b"),
                                   performance_external_reg(c(2,2,1,0),"c"),
                                   performance_external_reg(c(2,2,1,0),"d"),
                                   performance_external_reg(c(3,1,1,0),"b"),
                                   performance_external_reg(c(3,1,1,0),"c"),
                                   performance_external_reg(c(3,1,1,0),"d")
)
name = colnames(final_data_frame_esternal)
for(i  in dim(final_data_frame_esternal)[2]){
  final_data_frame_esternal[,i] <- as.numeric(final_data_frame_esternal[,i])
}




#Best model

min_WESP1 = min(final_data_frame_esternal$mean_wesp_out)
min_WESP_pqcomb1 = final_data_frame_esternal[final_data_frame_esternal$mean_wesp_out == min_WESP1,] #row 2 #c(2,2,1,0),"c"

min_rmse1 =  min(final_data_frame_esternal$mean_rmse_out)
min_rmse_pqcomb1 = final_data_frame_esternal[final_data_frame_esternal$mean_rmse_out == min_rmse1,] #row 4 #c(3,1,1,0),"b"

min_IS0.51 =  min(final_data_frame_esternal$IS_0.5)
min_IS0.5_pqcomb1 = final_data_frame_esternal[final_data_frame_esternal$IS_0.5 == min_IS0.51,]  #row 4 #c(3,1,1,0),"b"


min_IS0.81 =  min(final_data_frame_esternal$IS_0.8)
min_IS0.8_pqcomb1 = final_data_frame_esternal[final_data_frame_esternal$IS_0.8 == min_IS0.81,] #row 1 #c(2,2,1,0),"b"



min_avlength1 =  min(final_data_frame_esternal$AverageLength_0.8)
min_avlength_pqcomb1 = final_data_frame_esternal[final_data_frame_esternal$AverageLength_0.8 == min_avlength1,] #row 1#c(2,2,1,0),"b"


min_coverate1 = min(final_data_frame_esternal$CoverageRate_0.8)
min_min_coverate1 = final_data_frame_esternal[final_data_frame_esternal$CoverageRate_0.8 == min_coverate1,] #row 1 #c(2,2,1,0), "b


#So best with external regressor is c(2,2,1,0) + trade_size 
#                                   c(3,1,1,0) + trade_size




#Final comparison
tradesize_2210 <- final_data_frame_esternal[1,]
tradesize_3110 <- final_data_frame_esternal[4,]
pure_2310 <- best1
pure_2211 <- best2


FRAME <- rbind(tradesize_2210,tradesize_3110,pure_2310,pure_2211)
rownames(FRAME) <- c("tradesize_2210","tradesize_3110","pure_2310","pure_2211")
for(i  in dim(FRAME)[2]){
  FRAME[,i] <- as.numeric(FRAME[,i])
}

# In terms of wesp_out,                   pure_2211 
#             rmse_out,                   pure_2310 
#             IS.5.8(both),               tradesize_3110
#             coverage rate.5,            pure_2310 
#             coverage rate.8,            tradesize_3110
#             average length.5.8(both),   tradesize_2210






#Plot illustation for report 
#TS_SUB = TS_whole_data[1:15000,]
#base1 = ggplot(data = TS_whole_data, aes(x =TS_whole_data$trade_price_last1, y = TS_whole_data$trade_price))
#base1 + geom_point(colour = "brown") + xlab("Trade Price Last 1") + ylab("Actual Trade Price")

#base2 = ggplot(data = TS_whole_data, aes(x = TS_whole_data$curve_trade_diff))
#base2 + geom_bar(colour = "brown") + xlab("Trade Price Last 1") + ylab("Actual Trade Price")


#install.packages("Hmisc")
#library(Hmisc)

#latex(FRAME[c(2,3),])

#library(xtable)
#xtable(FRAME[c(2,3),c(1,2,3,4,10)],digits = 5)
#xtable(FRAME[c(2,3),c(5,6,7,8,9)],digits = 5)
#xtable(FRAME[c(2,3),c(7,8,9)],digits=5)

#ggplot(TS_whole_data, aes(x =TS_whole_data$current_coupon)) + geom_histogram(fill = "blue") + xlab("Current Coupon")
#ggplot(TS_whole_data, aes(x =TS_whole_data$current_coupon)) + geom_histogram(fill = "blue") + xlab("Current Coupon")


#ggplot(gather(TS_whole_data[,2:7]), aes(value)) + 
  #geom_histogram(bins = 20, colour = "#00AFBB", fill ="#00AFBB" ) + 
  #facet_wrap(~key, scales = 'free_x')




summary(TS_whole_data$trade_price)
hist(TS_whole_data$trade_price, xlab = "Trade Price", main = "Distribution of Trade Price")

dd = as.data.frame(TS_whole_data %>% group_by(bond_id) %>% count())
plot(dd$n ~ dd$bond_id)
dd$bond_id <- as.factor(dd$bond_id)

barplot(dd$n, xlab = "bond_id", ylab = "# of Recording")

hist(dd$n,breaks = seq(from = 1, to = 4132, by = 1), xlab = "Num of recording",main = "Frequency Distribution by Bond ID")
boxplot(dd$n,ylab = "Num of recording",main = "Frequence Boxplot")


summary(dd$n)




